#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include "selinux_internal.h"
#include <stdlib.h>
#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdio_ext.h>
#include "policy.h"

static int is_selinuxfs_available(void)
{
	char *buf = NULL;
	FILE *fp;
	int rc;
	size_t len;
	ssize_t num;

	/* init_selinuxmnt() gets called before this function.
	 * Assume that if a selinux file system is mounted, then
	 * it is available. */
	if (selinux_mnt)
		return 1;

	/* Drop back to detecting it the long way. */
	fp = fopen("/proc/filesystems", "r");
	if (!fp)
		return -1;

	__fsetlocking(fp, FSETLOCKING_BYCALLER);
	rc = 0;
	while ((num = getline(&buf, &len, fp)) >= 0) {
		if (strstr(buf, "selinuxfs")) {
			rc = 1;
			break;
		}
	}

	if (num < 0 && !feof(fp))
		rc = -1;

	free(buf);
	fclose(fp);
	return rc;
}

int is_selinux_enabled(void)
{
	int enabled;
	security_context_t con;

	/* If a selinux file system is available, assume that
	 * selinux is enabled. */
	enabled = is_selinuxfs_available();
	if (enabled <= 0)
		return enabled;

	/* If getcon_raw fails, selinux is still enabled.
	 * Consider it disabled only if no policy is loaded. */
	if (getcon_raw(&con) == 0) {
		if (!strcmp(con, "kernel"))
			enabled = 0;
		freecon(con);
	}
	return enabled;
}

hidden_def(is_selinux_enabled)

/*
 * Function: is_selinux_mls_enabled()
 * Return:   1 on success
 *	     0 on failure
 */
int is_selinux_mls_enabled(void)
{
	char buf[20], path[PATH_MAX];
	int fd, ret, enabled = 0;

	if (!selinux_mnt)
		return enabled;

	snprintf(path, sizeof path, "%s/mls", selinux_mnt);
	fd = open(path, O_RDONLY);
	if (fd < 0)
		return enabled;

	memset(buf, 0, sizeof buf);

	do {
		ret = read(fd, buf, sizeof buf - 1);
	} while (ret < 0 && errno == EINTR);
	close(fd);
	if (ret < 0)
		return enabled;

	if (!strcmp(buf, "1"))
		enabled = 1;

	return enabled;
}

hidden_def(is_selinux_mls_enabled)
